#ifndef OBOUFRONTA_H
  #define OBOUFRONTA_H

  #include <iostream.h>

  template<class T> class CObouFronta {
	private : 
		T *m_Pole;
		int m_Velikost;	// maximalni velikost (delka) fronty
		int m_Zacatek, m_Konec;	// kde je zacatek a konec fronty
		int m_Pocet;	// pocet prvku prave se nachazejicich ve fronte
			
	public :
		CObouFronta(int _Velikost);
		~CObouFronta();

		bool VlozZacatek(const T& _Vklad);
		bool VlozKonec(const T& _Vklad);
		bool VyjmiZacatek(T& _Vyber);
		bool VyjmiKonec(T& _Vyber);
		
		bool JePrazdna() { return ((0 == m_Pocet) ? true : false); }
		bool JePlna() { return ((m_Pocet == m_Velikost) ? true : false); }

		void Vypis();
  };
	
  template<class T> CObouFronta<T>::CObouFronta(int _Velikost)
  {
	  m_Velikost = -1;
	  m_Pocet = 0;
	  m_Zacatek = -1;
	  m_Konec = -1;	// abychom v metode Vloz dostali spravny konec po prvnim vlozeni
	  
	  m_Pole = new T[_Velikost];

	  if(m_Pole)
	  {
		  // pokud se podarilo nastavime velikost
		  m_Velikost = _Velikost;
	  }
  }

  template<class T> CObouFronta<T>::~CObouFronta()
  {
	  if(m_Pole)
	  {
		  delete[] m_Pole;
		  m_Pole = NULL;
		  m_Velikost = 0;
	  }
  }

  template<class T> bool CObouFronta<T>::VlozZacatek(const T& _Vklad)
  {
	  bool bNavrat = false;

	  if(m_Pole)
	  {
		  if(!JePlna())
		  {
			  if(m_Zacatek == 0)
			  {
				  // pokud zacatek dosahne zacatku pole, pak zaciname na poslednim prvku fronty
				  m_Zacatek = m_Velikost - 1;
			  }
			  else
			  {
				  // posun na novy zacatek, ale jen v pripade, ze nevkladame prvni prvek
				  if(!JePrazdna())
				  {
					  m_Zacatek--;
				  }
				  else
				  {
					  // pokud bylo pole prazdne, pak se zacatek i konec nachazi na stejnem miste
					  m_Zacatek = 0;
					  m_Konec = 0;
				  }
			  }

			  m_Pole[m_Zacatek] = _Vklad;
			  
			  m_Pocet++;	// pribyl prvek

			  bNavrat = true;
		  }
	  }

	  return bNavrat;
  }

  template<class T> bool CObouFronta<T>::VlozKonec(const T& _Vklad)
  {
	  bool bNavrat = false;

	  if(m_Pole)
	  {
		  if(!JePlna())
		  {
			  if(m_Konec == (m_Velikost - 1))
			  {
				  // pokud konec dosahne konce pole, pak koncime na prvnim prvku pole
				  m_Konec = 0;
			  }
			  else
			  {
				  // posun na novy konec, ale jen v pripade, ze nevkladame prvni prvek
				  if(!JePrazdna())
				  {
					  m_Konec++;
				  }
				  else
				  {
					  // pokud bylo pole prazdne, pak se zacatek i konec nachazi na stejnem miste
					  m_Zacatek = 0;
					  m_Konec = 0;
				  }
			  }

			  m_Pole[m_Konec] = _Vklad;
			  m_Pocet++;	// pribyl prvek

			  bNavrat = true;
		  }
	  }

	  return bNavrat;
  }

  template<class T> bool CObouFronta<T>::VyjmiZacatek(T& _Vyber)
  {
	  bool bNavrat = false;

	  if(m_Pole)
	  {
		  if(!JePrazdna())
		  {
			  _Vyber = m_Pole[m_Zacatek];
			  
			  if(m_Zacatek == m_Velikost - 1)
			  {
				  // pokud zacatek dosahne konce pole, pak zaciname zase na prvnim prvku
				  m_Zacatek = 0;
			  }
			  else
			  {
				  // posun na novy zacatek
				  m_Zacatek++;
			  }

			  m_Pocet--;	// ubyl prvek

			  bNavrat = true;
		  }
	  }

	  return bNavrat;
  }

  template<class T> bool CObouFronta<T>::VyjmiKonec(T& _Vyber)
  {
	  bool bNavrat = false;

	  if(m_Pole)
	  {
		  if(!JePrazdna())
		  {
			  _Vyber = m_Pole[m_Konec];
			  
			  if(m_Konec == 0)
			  {
				  // pokud konec dosahne zacatku pole, pak zaciname zase na poslednim prvku
				  m_Konec = m_Velikost - 1;
			  }
			  else
			  {
				  // posun na novy zacatek
				  m_Konec--;
			  }

			  m_Pocet--;	// ubyl prvek

			  bNavrat = true;
		  }
	  }

	  return bNavrat;
  }

  template <class T> void CObouFronta<T>::Vypis()
  {
	  if(m_Pole)
	  {
		  if(!JePrazdna())
		  {
			  cout << "Obsahuje " << m_Pocet << " objektu (volnych " << m_Velikost - (m_Pocet) << ")" << endl;
			  for(int i = 0; i < m_Velikost; i++)
			  {
				  cout << m_Pole[i] << endl;
			  }
		  }
		  else
		  {
			  cout << "Fronta je prazdna" << endl;
		  }
	  }
  }

#endif
